@@ -21,18 +21,20 @@ from __future__ import print_function
 from tensorflow.python.eager import context
 from tensorflow.python.framework import ops
 from tensorflow.python.keras import metrics
+from tensorflow.python.keras.utils import losses_utils
 from tensorflow.python.ops import array_ops
 from tensorflow.python.ops import control_flow_ops
 from tensorflow.python.ops import lookup_ops
 from tensorflow.python.ops import math_ops
 from tensorflow.python.ops import nn
 from tensorflow.python.ops import string_ops
-from tensorflow.python.ops.losses import losses
+from tensorflow.python.ops import weights_broadcast_ops
 from tensorflow_estimator.python.estimator import model_fn
 from tensorflow_estimator.python.estimator.canned import metric_keys
 from tensorflow_estimator.python.estimator.canned import prediction_keys
 from tensorflow_estimator.python.estimator.export import export_output
 from tensorflow_estimator.python.estimator.head import base_head
+from tensorflow_estimator.python.estimator.mode_keys import ModeKeys
 
 
 class BinaryClassHead(base_head.Head):
@@ -80,7 +82,7 @@ class BinaryClassHead(base_head.Head):
       reduce training loss over batch. Defaults to `SUM_OVER_BATCH_SIZE`, namely
       weighted sum of losses divided by `batch size * label_dimension`.
     loss_fn: Optional loss function.
-    name: name of the head. If provided, summary and metrics keys will be
+    name: Name of the head. If provided, summary and metrics keys will be
       suffixed by `"/" + name`. Also used as `name_scope` when creating ops.
   """
 
@@ -88,7 +90,7 @@ class BinaryClassHead(base_head.Head):
                weight_column=None,
                thresholds=None,
                label_vocabulary=None,
-               loss_reduction=losses.Reduction.SUM_OVER_BATCH_SIZE,
+               loss_reduction=losses_utils.ReductionV2.SUM_OVER_BATCH_SIZE,
                loss_fn=None,
                name=None):
     if label_vocabulary is not None and not isinstance(label_vocabulary,
@@ -100,14 +102,9 @@ class BinaryClassHead(base_head.Head):
     for threshold in thresholds:
       if (threshold <= 0.0) or (threshold >= 1.0):
         raise ValueError('thresholds not in (0, 1): {}.'.format((thresholds,)))
-    if (loss_reduction not in losses.Reduction.all() or
-        loss_reduction == losses.Reduction.NONE):
-      raise ValueError(
-          'Invalid loss_reduction: {}. See `tf.losses.Reduction` for valid '
-          'options.'.format(loss_reduction))
+    base_head.validate_loss_reduction(loss_reduction)
     if loss_fn:
       base_head.validate_loss_fn_args(loss_fn)
-
     self._weight_column = weight_column
     self._thresholds = thresholds
     self._label_vocabulary = label_vocabulary
@@ -158,7 +155,7 @@ class BinaryClassHead(base_head.Head):
   # Attributes for lookup tables in Eager execution. Note that for Graph
   # execution, the lookup tables are created on demand to make sure the lookup
   # table is in the same graph as its input tensors for `train` and `eval` of
-  # Estimator (as Estimator recreates graphes for `train`, `eval` and
+  # Estimator (as Estimator recreates graphs for `train`, `eval` and
   # `predict`).
   _cached_class_id_table = None
   _cached_class_string_table = None
@@ -230,8 +227,10 @@ class BinaryClassHead(base_head.Head):
       labels = self._processed_labels(logits, labels)
       unweighted_loss, weights = self._unweighted_loss_and_weights(
           logits, labels, features)
-      training_loss = losses.compute_weighted_loss(
-          unweighted_loss, weights=weights, reduction=self._loss_reduction)
+      training_loss = losses_utils.compute_weighted_loss(
+          unweighted_loss,
+          sample_weight=weights,
+          reduction=self._loss_reduction)
       regularization_loss = math_ops.add_n(
           regularization_losses) if regularization_losses is not None else None
       regularized_training_loss = (
@@ -255,7 +254,9 @@ class BinaryClassHead(base_head.Head):
     """
     pred_keys = prediction_keys.PredictionKeys
     valid_keys = [pred_keys.LOGITS, pred_keys.LOGISTIC, pred_keys.PROBABILITIES,
-                  pred_keys.CLASS_IDS, pred_keys.CLASSES]
+                  pred_keys.CLASS_IDS, pred_keys.CLASSES,
+                  pred_keys.ALL_CLASS_IDS, pred_keys.ALL_CLASSES]
+
     if keys:
       base_head.check_prediction_keys(keys, valid_keys)
     else:
@@ -287,6 +288,12 @@ class BinaryClassHead(base_head.Head):
           else:
             classes = string_ops.as_string(class_ids, name='str_classes')
           predictions[pred_keys.CLASSES] = classes
+      if pred_keys.ALL_CLASS_IDS in keys:
+        predictions[pred_keys.ALL_CLASS_IDS] = base_head.all_class_ids(
+            logits, n_classes=2)
+      if pred_keys.ALL_CLASSES in keys:
+        predictions[pred_keys.ALL_CLASSES] = base_head.all_classes(
+            logits, n_classes=2, label_vocabulary=self._label_vocabulary)
       return predictions
 
   def metrics(self, regularization_losses=None):
@@ -304,9 +311,9 @@ class BinaryClassHead(base_head.Head):
       eval_metrics[self._label_mean_key] = metrics.Mean(name=keys.LABEL_MEAN)
       eval_metrics[self._accuracy_baseline_key] = (
           metrics.Mean(name=keys.ACCURACY_BASELINE))
-      # TODO(b/118843532): create Keras metrics
-      # eval_metrics[self._auc_key] = metrics.Precision(name=keys.AUC)
-      # eval_metrics[self._auc_pr_key] = metrics.Precision(name=keys.AUC_PR)
+      # The default summation_method is "interpolation" in the AUC metric.
+      eval_metrics[self._auc_key] = metrics.AUC(name=keys.AUC)
+      eval_metrics[self._auc_pr_key] = metrics.AUC(curve='PR', name=keys.AUC_PR)
       if regularization_losses is not None:
         eval_metrics[self._loss_regularization_key] = metrics.Mean(
             name=keys.LOSS_REGULARIZATION)
@@ -355,6 +362,13 @@ class BinaryClassHead(base_head.Head):
         label_mean_metric.count - label_mean_metric.total)
     accuracy_baseline_metric.count = label_mean_metric.count
 
+  def _update_auc(self, auc_metric, labels, predictions, weights=None):
+    predictions = math_ops.to_float(predictions)
+    if weights is not None:
+      weights = weights_broadcast_ops.broadcast_weights(weights, predictions)
+    auc_metric.update_state(
+        y_true=labels, y_pred=predictions, sample_weight=weights)
+
   def update_metrics(self, eval_metrics, features, logits, labels,
                      regularization_losses=None):
     """Updates eval metrics. See `base_head.Head` for details."""
@@ -381,9 +395,12 @@ class BinaryClassHead(base_head.Head):
     base_head.update_metric_with_broadcast_weights(
         eval_metrics[self._label_mean_key], labels, weights)
     self._update_accuracy_baseline(eval_metrics)
-    # TODO(b/118843532): update Keras metrics
-    # eval_metrics[self._auc_key].update_state(...)
-    # eval_metrics[self._auc_pr_key].update_state(...)
+    self._update_auc(
+        auc_metric=eval_metrics[self._auc_key], labels=labels,
+        predictions=logistic, weights=weights)
+    self._update_auc(
+        auc_metric=eval_metrics[self._auc_pr_key], labels=labels,
+        predictions=logistic, weights=weights)
     if regularization_losses is not None:
       regularization_loss = math_ops.add_n(regularization_losses)
       eval_metrics[self._loss_regularization_key].update_state(
@@ -399,7 +416,8 @@ class BinaryClassHead(base_head.Head):
 
   def _create_tpu_estimator_spec(
       self, features, mode, logits, labels=None, optimizer=None,
-      train_op_fn=None, regularization_losses=None):
+      trainable_variables=None, train_op_fn=None, update_ops=None,
+      regularization_losses=None):
     """Returns an `EstimatorSpec`.
 
     Args:
@@ -407,21 +425,32 @@ class BinaryClassHead(base_head.Head):
         `SparseTensor` objects containing the values for that feature in a
         minibatch. Often to be used to fetch example-weight tensor.
       mode: Estimator's `ModeKeys`.
-      logits: logits `Tensor` with shape `[D0, D1, ... DN, 1]`. For many
+      logits: Logits `Tensor` with shape `[D0, D1, ... DN, 1]`. For many
         applications, the shape is `[batch_size, 1]`.
       labels: Labels integer or string `Tensor` with shape matching `logits`,
         namely `[D0, D1, ... DN, 1]` or `[D0, D1, ... DN]`. `labels` is required
         argument when `mode` equals `TRAIN` or `EVAL`.
-      optimizer: `Optimizer` instance to optimize the loss in TRAIN mode.
-        Namely, sets `train_op = optimizer.minimize(loss, global_step)`, which
-        updates variables and increments `global_step`.
+      optimizer: An `tf.keras.optimizers.Optimizer` instance to optimize the
+        loss in TRAIN mode. Namely, sets
+        `train_op = optimizer.get_updates(loss, trainable_variables)`,
+        which updates variables to minimize `loss`.
+      trainable_variables: A list or tuple of `Variable` objects to update to
+        minimize `loss`. In Tensorflow 1.x, by default these are the list of
+        variables collected in the graph under the key
+        `GraphKeys.TRAINABLE_VARIABLES`. As Tensorflow 2.x doesn't have
+        collections and GraphKeys, trainable_variables need to be passed
+        explicitly here.
       train_op_fn: Function that takes a scalar loss `Tensor` and returns
         `train_op`. Used if `optimizer` is `None`.
+      update_ops: A list or tuple of update ops to be run at training time. For
+        example, layers such as BatchNormalization create mean and variance
+        update ops that need to be run at training time. In Tensorflow 1.x,
+        these are thrown into an UPDATE_OPS collection. As Tensorflow 2.x
+        doesn't have collections, update_ops need to be passed explicitly here.
       regularization_losses: A list of additional scalar losses to be added to
         the training loss, such as regularization losses. These losses are
         usually expressed as a batch average, so for best results users need to
-        set `loss_reduction=SUM_OVER_BATCH_SIZE` or
-        `loss_reduction=SUM_OVER_NONZERO_WEIGHTS` when creating the head to
+        set `loss_reduction=SUM_OVER_BATCH_SIZE` when creating the head to
         avoid scaling errors.
 
     Returns:
@@ -435,14 +464,14 @@ class BinaryClassHead(base_head.Head):
       # Predict.
       pred_keys = prediction_keys.PredictionKeys
       predictions = self.predictions(logits)
-      if mode == model_fn.ModeKeys.PREDICT:
+      if mode == ModeKeys.PREDICT:
         probabilities = predictions[pred_keys.PROBABILITIES]
         logistic = predictions[pred_keys.LOGISTIC]
         classifier_output = base_head.classification_output(
             scores=probabilities, n_classes=2,
             label_vocabulary=self._label_vocabulary)
         return model_fn._TPUEstimatorSpec(  # pylint: disable=protected-access
-            mode=model_fn.ModeKeys.PREDICT,
+            mode=ModeKeys.PREDICT,
             predictions=predictions,
             export_outputs={
                 base_head.DEFAULT_SERVING_KEY: classifier_output,
@@ -456,10 +485,10 @@ class BinaryClassHead(base_head.Head):
           logits=logits, labels=labels, features=features, mode=mode,
           regularization_losses=regularization_losses)
       # Eval.
-      if mode == model_fn.ModeKeys.EVAL:
+      if mode == ModeKeys.EVAL:
         eval_metrics = self.metrics(regularization_losses=regularization_losses)
         return model_fn._TPUEstimatorSpec(  # pylint: disable=protected-access
-            mode=model_fn.ModeKeys.EVAL,
+            mode=ModeKeys.EVAL,
             predictions=predictions,
             loss=regularized_training_loss,
             eval_metrics=base_head.create_eval_metrics_tuple(
@@ -472,12 +501,20 @@ class BinaryClassHead(base_head.Head):
                 }))
       # Train.
       train_op = base_head.create_estimator_spec_train_op(
-          self._name, optimizer, train_op_fn, regularized_training_loss)
+          head_name=self._name,
+          optimizer=optimizer,
+          train_op_fn=train_op_fn,
+          update_ops=update_ops,
+          trainable_variables=trainable_variables,
+          regularized_training_loss=regularized_training_loss,
+          loss_reduction=self._loss_reduction)
     # Create summary.
     base_head.create_estimator_spec_summary(
-        regularized_training_loss, regularization_losses, self._summary_key)
+        regularized_training_loss=regularized_training_loss,
+        regularization_losses=regularization_losses,
+        summary_key_fn=self._summary_key)
     return model_fn._TPUEstimatorSpec(  # pylint: disable=protected-access
-        mode=model_fn.ModeKeys.TRAIN,
+        mode=ModeKeys.TRAIN,
         predictions=predictions,
         loss=regularized_training_loss,
         train_op=train_op)
